/*
 * $Id: config.cpp,v 1.2 2002/09/01 22:50:34 fredrik Exp $
 * 
 * Keramik KWin client configuration module
 *
 * Copyright (C) 2002 Fredrik Hglund <fredrik@kde.org>
 *
 * Based on the Quartz configuration module,
 *     Copyright (c) 2001 Karol Szwed <gallium@kde.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <kglobal.h>
#include <klocale.h>

#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>

#include "config.h"
#include "config.moc"

extern "C"
{
	QObject* allocate_config( KConfig* conf, QWidget* parent )
	{
		return ( new ThinKeramikConfig( conf, parent ) );
	}
}


/* NOTE: 
 * 'conf' 	is a pointer to the kwindecoration modules open kwin config,
 *			and is by default set to the "Style" group.
 *
 * 'parent'	is the parent of the QObject, which is a VBox inside the
 *			Configure tab in kwindecoration
 */

ThinKeramikConfig::ThinKeramikConfig( KConfig* conf, QWidget* parent )
	: QObject( parent )
{
	KGlobal::locale()->insertCatalogue("kwin_thinkeramik_config");
	c = new KConfig( "kwinthinkeramikrc" );
	
	ui = new ThinKeramikConfigUI( parent );
	connect( ui->showAppIcons,    SIGNAL(clicked()), SIGNAL(changed()) );
	//connect( ui->largeGrabBars,   SIGNAL(clicked()), SIGNAL(changed()) );
	connect( ui->captionType, SIGNAL(clicked(int)), this, SIGNAL(changed()) );
	connect( ui->useShadowedText, SIGNAL(clicked()), SIGNAL(changed()) );
	connect( ui->buttonType, SIGNAL(clicked(int)), this, SIGNAL(changed()) );
	connect( ui->drawTtlBdr, SIGNAL(clicked()), SIGNAL(changed()) );
	connect( ui->useTBColor, SIGNAL(clicked()), SIGNAL(changed()) );

	load( conf );
	ui->show();
}


ThinKeramikConfig::~ThinKeramikConfig()
{
	delete ui;
	delete c;
}


// Loads the configurable options from the kwinrc config file
// It is passed the open config from kwindecoration to improve efficiency
void ThinKeramikConfig::load( KConfig* )
{
	c->setGroup("General");
	ui->showAppIcons->setChecked( c->readBoolEntry("ShowAppIcons", true) );
	//ui->largeGrabBars->setChecked( c->readBoolEntry("LargeGrabBars", false) );
	ui->useShadowedText->setChecked( c->readBoolEntry("UseShadowedText", true) );

	bool useSmallCaptions = c->readBoolEntry("SmallCaptionBubbles", false);
        QString value = c->readEntry("CaptionType", "normal");
	if ((value.compare("small")) == 0 || useSmallCaptions) {
	      ui->captionType->setButton(1);
	} else if ((value.compare("none")) == 0) {
	      ui->captionType->setButton(2);
	} else {
	      ui->captionType->setButton(0);
	}
        value = c->readEntry("ButtonType", "Normal");
	if ((value.compare("EdgedButton")) == 0) {
	      ui->buttonType->setButton(1);
	} else if ((value.compare("NoBackground")) == 0) {
	      ui->buttonType->setButton(2);
	} else {
	      ui->buttonType->setButton(0);
	}
	ui->drawTtlBdr->setChecked( c->readBoolEntry("DrawTitleBorder", false) );
	ui->useTBColor->setChecked( c->readBoolEntry("UseTitleBlendColor", false) );

}


// Saves the configurable options to the kwinrc config file
void ThinKeramikConfig::save( KConfig* )
{
	c->setGroup( "General" );
	c->writeEntry( "ShowAppIcons", ui->showAppIcons->isChecked() );
	//c->writeEntry( "LargeGrabBars", ui->largeGrabBars->isChecked() );
	c->writeEntry( "UseShadowedText", ui->useShadowedText->isChecked() );

	QRadioButton *capType = (QRadioButton*)ui->captionType->selected();
	QString capTypeSelected = QString(capType->name());
        if (capType) c->writeEntry("CaptionType", capTypeSelected);
	if ((capTypeSelected.compare("small")) == 0 ) {
	      c->writeEntry( "SmallCaptionBubbles", true );
	} else {
	      c->writeEntry( "SmallCaptionBubbles", false );
	}
	QRadioButton *btnType = (QRadioButton*)ui->buttonType->selected();
        if (btnType) c->writeEntry("ButtonType", QString(btnType->name()));
	c->writeEntry( "DrawTitleBorder", ui->drawTtlBdr->isChecked() );
	c->writeEntry( "UseTitleBlendColor", ui->useTBColor->isChecked() );

	c->sync();
}


// Sets UI widget defaults which must correspond to style defaults
void ThinKeramikConfig::defaults()
{
	ui->showAppIcons->setChecked( true );
	//ui->largeGrabBars->setChecked( false );
	ui->useShadowedText->setChecked( true );

        QRadioButton *capType =
            (QRadioButton*)ui->captionType->child("normal");
        QRadioButton *btnType =
            (QRadioButton*)ui->buttonType->child("NormalButton");
        if (btnType) btnType->setChecked(true);
	ui->drawTtlBdr->setChecked( false );
	ui->useTBColor->setChecked( false );

	
	emit changed();
}

// vim: set noet ts=4 sw=4:
