/*
 * $Id: keramik.h,v 1.12 2003/12/15 15:10:16 lunakl Exp $
 *
 * Keramik KWin client (version 0.8)
 *
 * Copyright (C) 2002 Fredrik Hglund <fredrik@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __THINKERAMIK_H
#define __THINKERAMIK_H

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>

#include "tiles.h"

class QSpacerItem;

namespace ThinKeramik {

	enum TilePixmap  { TitleLeft=0, TitleCenter, TitleRight,
	                   CaptionSmallLeft, CaptionSmallCenter, CaptionSmallRight,
	                   CaptionLargeLeft, CaptionLargeCenter, CaptionLargeRight,
					   GrabBarLeft, GrabBarCenter, GrabBarRight,
	                   BorderLeft, BorderRight, NumTiles };

	enum Button      { MenuButton=0, OnAllDesktopsButton, HelpButton, MinButton,
	                   MaxButton, CloseButton, NumButtons };

	enum ButtonDeco  { Menu=0, OnAllDesktops, NotOnAllDesktops, Help, Minimize, Maximize,
	                   Restore, Close, NumButtonDecos };

	struct SettingsCache
	{
		bool largeGrabBars:1;
		bool smallCaptionBubbles:1;
		bool noCaptionBubbles:1;
		bool useTBColor:1;
	};

	class ThinKeramikHandler : public KDecorationFactory
	{
		public:
			ThinKeramikHandler();
			~ThinKeramikHandler();

			virtual QValueList< BorderSize > borderSizes() const;
			virtual bool reset( unsigned long changed );
                        virtual KDecoration* createDecoration( KDecorationBridge* );

			bool showAppIcons() const        { return showIcons; }
			bool useShadowedText() const     { return shadowedText; }
			bool largeCaptionBubbles() const { return !(smallCaptionBubbles || noCaptionBubbles); }
			QString btnTyp() const     { return buttonType; }
			bool drawTitleBorder() const     { return drawTBdr; }

			int titleBarHeight( bool large ) const {
				return ( large ? activeTiles[CaptionLargeCenter]->height()
						: activeTiles[CaptionSmallCenter]->height() );
			}

			int grabBarHeight() const
				{ return activeTiles[GrabBarCenter]->height(); }

			const QPixmap *roundButton() const  { return titleButtonRound; }
			const QPixmap *roundButton2() const  { return titleButtonRound2; }
			const QPixmap *roundButton3() const  { return titleButtonRound3; }
			const QPixmap *squareButton() const { return titleButtonSquare; }
			const QPixmap *squareButton2() const { return titleButtonSquare2; }
			const QPixmap *squareButton3() const { return titleButtonSquare3; }
			const QBitmap *buttonDeco( ButtonDeco deco ) const
				{ return buttonDecos[ deco ]; }

			inline const QPixmap *tile( TilePixmap tilePix, bool active ) const;

		private:
			void readConfig();
			void createPixmaps();
			void destroyPixmaps();

			void addWidth  (int width,  QPixmap *&pix, bool left, QPixmap *bottomPix);
			void addHeight (int height, QPixmap *&pix);
			void flip( QPixmap *&, QPixmap *& );
			void flip( QPixmap *& );
			void pretile( QPixmap *&, int, Qt::Orientation );
			QPixmap *composite( QImage *, QImage * );
			QImage  *loadImage( const QString &, const QColor & );
			QPixmap *loadPixmap( const QString &, const QColor & );

			bool showIcons:1, shadowedText:1,
				smallCaptionBubbles:1, largeGrabBars:1;

	                bool noCaptionBubbles:1;
			QString buttonType;
			bool drawTBdr:1;
			bool useTBColor:1;

			SettingsCache *settings_cache;
			ThinKeramikImageDb *imageDb;

			QPixmap *activeTiles[ NumTiles ];
			QPixmap *activeTiles2[ NumTiles ];
			QPixmap *inactiveTiles[ NumTiles ];
			QPixmap *inactiveTiles2[ NumTiles ];
			QBitmap *buttonDecos[ NumButtonDecos ];

			QPixmap *titleButtonRound, *titleButtonSquare;
			QPixmap *titleButtonRound2, *titleButtonSquare2;
			QPixmap *titleButtonRound3, *titleButtonSquare3;

	}; // class ThinKeramikHandler

	class ThinKeramikClient;
	class ThinKeramikButton : public QButton
	{
		public:
			ThinKeramikButton( ThinKeramikClient *, const char *, Button, const QString &, const int realizeBtns = LeftButton );
			~ThinKeramikButton();

			int lastButton() const { return lastbutton; }

		private:
			void enterEvent( QEvent * );
			void leaveEvent( QEvent * );
			void mousePressEvent( QMouseEvent * );
			void mouseReleaseEvent( QMouseEvent * );
			void drawButton( QPainter * );

		private:
			ThinKeramikClient *client;
			Button button;
			bool hover;
			int lastbutton;
			int realizeButtons;
	}; // class ThinKeramikButton


	class ThinKeramikClient : public KDecoration
	{
		Q_OBJECT

		public:

			ThinKeramikClient( KDecorationBridge* bridge, KDecorationFactory* factory );
			~ThinKeramikClient();
                        virtual void init();
			virtual void reset( unsigned long changed );
			virtual Position mousePosition( const QPoint& p ) const;
		    	virtual void borders( int& left, int& right, int& top, int& bottom ) const;
			virtual void resize( const QSize& s );
			virtual QSize minimumSize() const;
			virtual bool eventFilter( QObject* o, QEvent* e );
			virtual void activeChange();
			virtual void captionChange();
                        virtual void maximizeChange();
                        virtual void desktopChange();
                        virtual void shadeChange() {};

		private:
			void createLayout();
			void addButtons( QBoxLayout*, const QString & );
			void updateMask(); // FRAME
			void updateCaptionBuffer();
			void iconChange();
			void resizeEvent( QResizeEvent *); // FRAME
			void paintEvent( QPaintEvent *); // FRAME
			void mouseDoubleClickEvent( QMouseEvent * ); // FRAME
			int width() const { return widget()->width(); }
			int height() const { return widget()->height(); }

			void calculateCaptionRect();

			inline bool maximizedVertical() const {
				return ( maximizeMode() & MaximizeVertical );
			}

		private slots:
			void menuButtonPressed();
			void slotMaximize();

		private:
			QSpacerItem   *topSpacer, *titlebar;
			ThinKeramikButton *button[ NumButtons ];
			QRect          captionRect;
			QPixmap        captionBuffer;
			QPixmap       *activeIcon, *inactiveIcon;
			bool           captionBufferDirty:1, maskDirty:1;
			bool           largeCaption:1, largeTitlebar:1;
	}; // class ThinKeramikClient

} // namespace ThinKeramik

#endif // ___THINKERAMIK_H

// vim: set noet ts=4 sw=4:
